use palette::encoding::srgb::Srgb;

use rgb::RGB8;

pub trait IntoRgb8 {
    fn into_rgb8(self) -> RGB8;
}

impl IntoRgb8 for RGB8 {
    fn into_rgb8(self) -> RGB8 {
        self
    }
}

impl IntoRgb8 for palette::Hsv<Srgb, u8> {
    fn into_rgb8(self) -> RGB8 {
        if self.saturation == 0 {
            return RGB8::new(self.value, self.value, self.value);
        }

        let region = self.hue.into_inner() / 43;
        let remainder = (self.hue.into_inner() - (region * 43)) * 6;

        let p = self.value.wrapping_mul(255 - self.saturation).wrapping_shr(8);
        let q = (self.value.wrapping_mul(255 - ((self.saturation.wrapping_mul(remainder)).wrapping_shr(8)))).wrapping_shr(8);
        let t = (self.value.wrapping_mul(255 - ((self.saturation.wrapping_mul(255 - remainder)).wrapping_shr(8)))).wrapping_shr(8);

        match region {
            0 => RGB8::new(self.value, t, p),
            1 => RGB8::new(q, self.value, p),
            2 => RGB8::new(p, self.value, t),
            3 => RGB8::new(p, q, self.value),
            4 => RGB8::new(t, p, self.value),
            _ => RGB8::new(self.value, p, q)
        }
    }
}

#[cfg(feature="embedded-graphics")]
mod embedded_graphics {
    use embedded_graphics::pixelcolor::RgbColor;
    use embedded_graphics::pixelcolor::PixelColor;
    use embedded_graphics::pixelcolor::raw::RawU8;

    impl RgbColor for RGB8 {
        fn r(&self) -> u8 { self.red }
        fn g(&self) -> u8 { self.green }
        fn b(&self) -> u8 { self.blue }

        const MAX_R: u8 = 255;
        const MAX_G: u8 = 255;
        const MAX_B: u8 = 255;

        const BLACK: Self = Self::new(0, 0, 0);
        const WHITE: Self = Self::new(255, 255, 255);

        const RED: Self = Self::new(255, 0, 0);
        const GREEN: Self = Self::new(0, 255, 0);
        const BLUE: Self = Self::new(0, 0, 255);

        const YELLOW: Self = Self::new(255, 0, 0);
        const CYAN: Self = Self::new(0, 255, 0);
        const MAGENTA: Self = Self::new(0, 0, 255);
    }

    impl PixelColor for RGB8 {
        type Raw = RawU8;
    }
}
